import { 
  select, 
  json, 
  zoom,
  forceSimulation, 
  forceLink,
  forceManyBody, 
  forceCenter,
  drag,
  event,
  continuous,
  scaleLinear
} from 'd3';
import {createContextMenu} from "./contextMenu.js";


// Visualisation Variables
const width = 960;
const height = 500;
const textPadding = 8;
const corner = 5;
const ent_colour = '#03396c';
const ent_t_colour = 'white';
const att_colour = '#ff4d00';
const att_t_colour = 'white';
const rel_colour = '#007777';
const rel_t_colour = 'black';

// RMB Menu Items
const menuItems = [
    {
      title: 'First action',
      action: (d) => {
        // TODO: add any action you want to perform
        console.log(d);
      }
    },
    {
      title: 'Second action',
      action: (d) => {
        // TODO: add any action you want to perform
        console.log(d);
      }
    }
  ];

const svg = d3.select('.mysvg')
	.append('svg')
	.attr('class', 'SVG')
	.attr("viewBox", [0, 0, width, height])
	.call(d3.zoom().on("zoom", function () {
            svg.attr("transform", d3.event.transform)
        }))
        .on("dblclick.zoom", null)
    		.append("g");

// Main rendering function
const render = data => {
    // Setup Basic Variables
    const nodes = data.nodes;
    const edges = data.links;
    const gTypes = data.details;
    // setup group for all graph objects
    const g = svg.append('g'). attr('class', 'graph');

    // setup the simulation forces
    var simulation = d3.forceSimulation()
        .force('link', d3.forceLink()
                        .id(function(d) { return d.id; })
                        .distance(120))
        .force('charge', d3.forceManyBody(-400))
        .force('center', d3.forceCenter(width / 2, height / 2));  

    // Setup Links
    const edge = g.append('g')
        .attr('class', 'edges')
        .selectAll("line")
        .data(edges)
        .enter()
          .append('line')
          .attr('stroke-width', '1px')
    			.attr('stroke', 'black');  

    // Setup Nodes Without Shape First
    const node = g.append('g')
    		.attr('class', 'nodes')
    		.selectAll('g') 
        .data(nodes)    
        .enter()
        	.append('g')
    			.attr('class', d => d.type)
    			.attr('id', d => d.id);

		// Setup Shapes Variable, so it draws before text
		const shape = node.append("rect") 
            .attr('class', d => d.G_name)
    				.attr('id', d => d.id)
    			.on('contextmenu', (d, i) => {
            createContextMenu(d, menuItems, width, height, 'SVG');
          });
  
  	// setup the text, headings, without descriptions, to start for each node
    var headings = node.append('text')
        .text(d => d.type == 'attribute' 
              ? (d.G_name + ': ' + d.value) 
              : (d.G_name + ': ' + d.id))
        .attr('id', d => 'heading-' + d.id)
        .style('font-size', 10)
        .style('text-anchor', 'middle')
        .style('color', d => text_colour(d))
        .style('fill', d => text_colour(d))
        .style("pointer-events", "none");

		// setup heading colours for attribute, entity and relation
    function text_colour(d) {
      	let colour = 'black'
        switch (d.type) {
            case 'attribute':
                colour = att_t_colour
                break;            
            case 'entity':
                colour = ent_t_colour
                break;
            case 'relation':
                colour = rel_t_colour
                break;
            default:
                colour = 'black'
                break;
        }
        return colour;
    }

		// draw the shape depnding on heading width, node-type, and color  
    shape.attr('width', d => nodeWidth(d))
        .attr('height', d => d.height)
        .attr('rx', d => d.corner)
        .attr('ry', d=> d.corner)     
        .attr('x', d => d.x)
        .attr('y', d => d.y)   
        .attr("class", d => d.G_name) 
        .style("fill", d => d.colour)
  			.classed("fixed", d => d.fx !== undefined);         

    // function to set dimensions and colours for shapes
    function nodeWidth(d) {
        let headingid = '#heading-' + d.id;
        let headingbbox = d3.select(headingid).node().getBBox();    

        d.width = d3.max([5, headingbbox.width]) + (textPadding * 2)
        d.height = headingbbox.height  + textPadding * 2        

        // attributes and entities rounded rectangle, relation is circle
        switch (d.type) {
            case 'attribute':
                d.corner = corner
                d.colour = att_colour
                d.x = headingbbox.x - textPadding
                d.y = headingbbox.y - textPadding
                break;

            case 'entity':
                d.corner = corner
                d.colour = ent_colour
                d.x = headingbbox.x - textPadding
                d.y = headingbbox.y - textPadding
                break;

            case 'relation':
                d.width = 10
                d.height = 10
                d.corner = 5
                d.colour = rel_colour
                d.x = headingbbox.x + (headingbbox.width)/2 - d.width/2
                d.y = headingbbox.y + d.height/2
                break;

            default:
                break;
        }
        return d.width;
    };
		
  	// Simulation Run Time
  	simulation
        .nodes(nodes)
        .on("tick", ticked);

    simulation.force("link")
        .links(edges);

    function ticked() {
        edge
            .attr("x1", function(d) { return d.source.x; })
            .attr("y1", function(d) { return d.source.y; })
            .attr("x2", function(d) { return d.target.x; })
            .attr("y2", function(d) { return d.target.y; });

        node
            .attr("transform", function(d) {
            	return "translate(" + d.x + "," + d.y + ")";
            })
    }


    // Setup the svg zooming
    
  	
  	// Setup the Dragging
  	const drag = d3
      .drag()
      .on("start", dragstart)
      .on("drag", dragged); 
  
  	node.call(drag).on("dblclick", click);
  
  	function click(event, d) {
      delete d.fx;
      delete d.fy;
      d3.select(this).classed("fixed", false);
      simulation.alpha(1).restart();
    }

    function dragstart() {
      d3.select(this).classed("fixed", true);
    }

    function dragged(event, d) {
      d.fx = event.x;
      d.fy = event.y;
      simulation.alpha(1).restart();
    }
    
  

}; // end of render

// load data
d3.json('graph.json', function (error, data) {
	render(data);
});